@@ -58,6 +58,7 @@ src_header_files = Split('''
     Dimension.h
     DispatchResult.h
     DocIterator.h
+    Enchant.h
     Encoding.h
     ErrorList.h
     Exporter.h
@@ -244,6 +245,7 @@ src_post_files = Split('''
 
 src_extra_src_files = Split('''
     ASpell.cpp
+    Enchant.cpp
     ISpell.cpp
     main.cpp
     PSpell.cpp
@@ -1175,6 +1177,7 @@ po_extra_files = Split('''
     de.po
     en@boldquot.header
     en@quot.header
+    el.po
     en.po
     es.po
     eu.po
@@ -1293,6 +1296,7 @@ lib_templates_files = Split('''
     dinbrief.lyx
     docbook_article.lyx
     elsarticle.lyx
+    es_beamer-conference-ornate-20min.lyx
     fr_beamer-conference-ornate-20min.lyx
     g-brief-de.lyx
     g-brief-en.lyx
@@ -1311,6 +1315,11 @@ lib_templates_files = Split('''
     revtex.lyx
     revtex4.lyx
     slides.lyx
+    svjour3.lyx
+    svmono_book.lyx
+    svmult_appendix.lyx
+    svmult_author.lyx
+    svmult_editor.lyx
 ''')
 
 
@@ -1484,6 +1493,7 @@ lib_images_files = Split('''
     vc-check-out.png
     vc-locking-toggle.png
     vc-register.png
+    vc-repo-update.png
     vc-revert.png
 ''')
 
@@ -1587,6 +1597,7 @@ lib_images_math_files = Split('''
     diagdown.png
     diagup.png
     diamond.png
+    diamond2.png
     diamondsuit.png
     digamma.png
     div.png
@@ -2262,7 +2273,12 @@ lib_doc_it_clipart_files = Split('''
 
 
 lib_doc_ja_files = Split('''
+    Customization.lyx
+    DummyDocument1.lyx
+    DummyDocument2.lyx
+    EmbeddedObjects.lyx
     Extended.lyx
+    Formula-numbering.lyx
     Intro.lyx
     LaTeXConfig.lyx
     Math.lyx
@@ -2273,10 +2289,12 @@ lib_doc_ja_files = Split('''
 
 
 lib_doc_ja_clipart_files = Split('''
+    ChildDocumentQt4.png
+    ERT.png
+    ExternalMaterialQt4.png
     floatQt4.png
     footnoteQt4.png
     referenceQt4.png
-    ERT.png
     ToolbarEnvBox.png
 ''')
 
@@ -2345,6 +2363,12 @@ lib_doc_uk_clipart_files = Split('''
 ''')
 
 
+lib_doc_zhCN_files = Split('''
+    Intro.lyx
+    Tutorial.lyx
+''')
+
+
 lib_examples_files = Split('''
     aa_sample.lyx
     aas_sample.lyx
@@ -2396,6 +2420,8 @@ lib_examples_files = Split('''
     serial_letter3.lyx
     simplecv.lyx
     splash.lyx
+    tufte-book.lyx
+    tufte-handout.lyx
     xyfigure.png
     xypic.lyx
 ''')
@@ -2431,6 +2457,7 @@ lib_examples_de_files = Split('''
     serienbrief2.lyx
     serienbrief3.lyx
     splash.lyx
+    tufte-handout.lyx
 ''')
 
 
@@ -2444,15 +2471,21 @@ lib_examples_fr_files = Split('''
     Braille.lyx
     exemple_brut.lyx
     exemple_lyxifie.lyx
+    exemple-powerdot.lyx
     Foils.lyx
+    linguistics.lyx
     ListesPuces.lyx
     multicol.lyx
+    seminar.lyx
     simplecv.lyx
     splash.lyx
+    xyfigure.png
+    xypic.lyx
 ''')
 
 
 lib_examples_es_files = Split('''
+    beamer-conference-ornate20min.lyx
     Braille.lyx
     ejemplo_con_lyx.lyx
     ejemplo_sin_lyx.lyx
@@ -2460,6 +2493,8 @@ lib_examples_es_files = Split('''
     linguistics.lyx
     multicol.lyx
     splash.lyx
+    tufte-book.lyx
+    tufte-handout.lyx
 ''')
 
 
@@ -2504,7 +2539,11 @@ lib_examples_it_files = Split('''
 
 lib_examples_ja_files = Split('''
     Braille.lyx
+    linguistics.lyx
+    multicol.lyx
     splash.lyx
+    xyfigure.png
+    xypic.lyx
 ''')
 
 
@@ -2537,6 +2576,11 @@ lib_examples_ru_files = Split('''
 ''')
 
 
+lib_examples_sk_files = Split('''
+    splash.lyx
+''')
+
+
 lib_examples_sl_files = Split('''
     primer_lyxan.lyx
     primer_surov.lyx
@@ -2605,6 +2649,7 @@ lib_layouts_files = Split('''
     docbook-chapter.layout
     docbook-section.layout
     docbook.layout
+    doublecol-new.layout
     dtk.layout
     egs.layout
     elsart.layout
@@ -2664,15 +2709,20 @@ lib_layouts_files = Split('''
     sigplanconf.layout
     simplecv.layout
     singlecol.layout
+    singlecol-new.layout
     slides.layout
     spie.layout
     svglobal.layout
     svglobal3.layout
     svjog.layout
+    svmono.layout
+    svmult.layout
     svprobth.layout
     tarticle.layout
     tbook.layout
     treport.layout
+    tufte-book.layout
+    tufte-handout.layout
 ''')
 
 
@@ -2717,6 +2767,8 @@ lib_layouts_inc_files = Split('''
     svjour.inc
     theorems.inc
     theorems-ams.inc
+    theorems-ams-bytype.inc
+    theorems-bytype.inc
     theorems-order.inc
     theorems-proof.inc
     theorems-starred.inc
@@ -2727,14 +2779,22 @@ lib_layouts_inc_files = Split('''
 lib_layouts_module_files = Split('''
     braille.module
     endnotes.module
+    eqs-within-sections.module
+    figs-within-sections.module
     foottoend.module
     hanging.module
     linguistics.module
     logicalmkup.module
     minimalistic.module
+    tabs-within-sections.module
+    theorems-ams-bytype.module
     theorems-ams-extended.module
+    theorems-ams-extended-bytype.module
     theorems-ams.module
+    theorems-bytype.module
+    theorems-chap-bytype.module
     theorems-chap.module
+    theorems-sec-bytype.module
     theorems-sec.module
     theorems-starred.module
     theorems-std.module
